@@ -3,10 +3,15 @@
 set -e
 
 if [ -z "$CMAKE" ]; then
-	echo "CMAKE isn't set, using default 'cmake'"
 	CMAKE=cmake
+fi
+
+if [ -x "$(which $CMAKE)" ]; then
+	echo CMAKE=$CMAKE
 else
-	echo "Using $CMAKE"
+	echo "$CMAKE doesn't exist"
+	ls -lah $(dirname $CMAKE)
+	exit 1
 fi
 
 SOURCEDIR=$(readlink -f $(dirname $0)/..)
