@@ -19,6 +19,8 @@ public:
 	virtual void ClearStartBoxes(){};
 	virtual void AddStartBox(int /*posx*/, int /*posy*/, int /*w*/, int /*h*/){};
 	virtual void Notify(){};
+	virtual void Promote(){};
+	virtual void Ring(){};
 	virtual void Start(){};
 	void SetBattle(IBattle* battle);
 
@@ -45,6 +47,8 @@ public:
 	void ClearStartBoxes() override;
 	void AddStartBox(int posx, int posy, int w, int h) override;
 	void Notify() override;
+	void Promote() override;
+	void Ring() override;
 	void Start() override;
 };
 
@@ -60,6 +64,8 @@ public:
 	void ClearStartBoxes() override;
 	void AddStartBox(int posx, int posy, int w, int h) override;
 	void Notify() override;
+	void Promote() override;
+	void Ring() override;
 	void Start() override;
 };
 
