@@ -25,7 +25,6 @@
 #include "Logger.h"
 #include "Downloader/Mirror.h"
 #include "Downloader/CurlWrapper.h"
-#include "lib/base64/base64.h"
 
 static size_t WriteMemoryCallback(void* contents, size_t size, size_t nmemb,
 				  void* userp)
@@ -91,7 +90,7 @@ static std::string getRequestUrl(const std::string& name,
 	if (cat != DownloadEnum::CAT_NONE) {
 		url += "category=" + DownloadEnum::getCat(cat) + std::string("&");
 	}
-	return url + std::string("torrent=true&springname=") + name;
+	return url + std::string("springname=") + name;
 }
 
 bool CHttpDownloader::ParseResult(const std::string& /*name*/,
@@ -161,10 +160,6 @@ bool CHttpDownloader::ParseResult(const std::string& /*name*/,
 			}
 		}
 
-		if (resfile["torrent"].isString()) {
-			const std::string torrent = base64_decode(resfile["torrent"].asString());
-			fileSystem->parseTorrent(&torrent[0], torrent.size(), dl);
-		}
 		if (resfile["version"].isString()) {
 			const std::string& version = resfile["version"].asString();
 			dl->version = version;
@@ -280,7 +275,7 @@ std::vector<unsigned int>
 CHttpDownloader::verifyAndGetNextPieces(CFile& file, IDownload* download)
 {
 	std::vector<unsigned int> pieces;
-	if (download->state == IDownload::STATE_FINISHED) {
+	if (download->isFinished()) {
 		return pieces;
 	}
 	// verify file by md5 if pieces.size == 0
@@ -346,7 +341,7 @@ bool CHttpDownloader::setupDownload(DownloadData* piece)
 {
 	std::vector<unsigned int> pieces =
 	    verifyAndGetNextPieces(*(piece->download->file), piece->download);
-	if (piece->download->state == IDownload::STATE_FINISHED)
+	if (piece->download->isFinished())
 		return false;
 	if (piece->download->file) {
 		piece->download->size = piece->download->file->GetPieceSize(-1);
@@ -540,7 +535,7 @@ static void CleanupDownloads(std::list<IDownload*>& download,
 		    curl_easy_getinfo(downloads[i]->curlw->GetHandle(), CURLINFO_FILETIME, &timestamp) == CURLE_OK) {
 			if (timestamp > 0) {
 				// decrease local timestamp if download failed to force redownload next time
-				if (downloads[i]->download->state != IDownload::STATE_FINISHED)
+				if (!downloads[i]->download->isFinished())
 					timestamp--;
 				downloads[i]->download->file->SetTimestamp(timestamp);
 			}
@@ -553,13 +548,24 @@ static void CleanupDownloads(std::list<IDownload*>& download,
 	downloads.clear();
 }
 
+void VerifySinglePieceDownload(IDownload& dl)
+{
+	if ((dl.hash == nullptr) || (dl.file == nullptr))
+		return;
+
+	if (dl.file->Hash(*dl.hash)) {
+		dl.state = IDownload::STATE_FINISHED;
+	}
+
+}
+
 bool CHttpDownloader::download(std::list<IDownload*>& download,
 			       int max_parallel)
 {
 	std::vector<DownloadData*> downloads;
 	CURLM* curlm = curl_multi_init();
 	for (IDownload* dl : download) {
-		if (dl->state == IDownload::STATE_FINISHED) {
+		if (dl->isFinished()) {
 			continue;
 		}
 		if (dl->dltype != IDownload::TYP_HTTP) {
@@ -572,7 +578,7 @@ bool CHttpDownloader::download(std::list<IDownload*>& download,
 			1, std::min((int)dl->pieces.size(),
 				    dl->getMirrorCount()))); // count of parallel downloads
 		if (dl->getMirrorCount() <= 0) {
-			LOG_ERROR("No mirrors found");
+			LOG_WARN("No mirrors found");
 			return false;
 		}
 		LOG_DEBUG("Using %d parallel downloads", count);
@@ -641,6 +647,10 @@ bool CHttpDownloader::download(std::list<IDownload*>& download,
 		select(count + 1, &rSet, &wSet, &eSet, &t);
 	}
 
+	for (IDownload* download: download) {
+		VerifySinglePieceDownload(*download);
+	}
+
 	LOG("\n");
 
 	if (!aborted) {
