@@ -5,6 +5,7 @@
 #include <stdio.h>
 #include <curl/curl.h>
 #include <stdlib.h>
+#include <errno.h>
 
 #include "Sdp.h"
 #include "RapidDownloader.h"
@@ -67,7 +68,7 @@ bool CSdp::downloadSelf(IDownload* /*dl*/)
 
 
 	if (!fileSystem->Rename(tmpFile, sdpPath)) {
-		LOG_ERROR("Couldn't rename %s to %s", tmpFile.c_str(), sdpPath.c_str());
+		LOG_ERROR("Couldn't rename %s to %s: %s", tmpFile.c_str(), sdpPath.c_str(), strerror(errno));
 		return false;
 	}
 
