@@ -45,7 +45,7 @@ void test_odb_packed_one__read_header_0(void)
 		git_oid id;
 		git_odb_object *obj;
 		size_t len;
-		git_otype type;
+		git_object_t type;
 
 		cl_git_pass(git_oid_fromstr(&id, packed_objects_one[i]));
 
