@@ -94,17 +94,16 @@ typedef void (*IDownloaderProcessUpdateListener)(int done, int size);
 
 extern void SetDownloadListener(IDownloaderProcessUpdateListener listener);
 
-/*
- * Calculate hash and return it in base64 format.
- * Accepted values for type are:
- *   0 - md5
-*/
-extern char* CalcHash(const char* str, int size, int type);
-
 /**
 * abort all downloads - must be called before shutting down,
 * all downloads must return before calling shutdown
 */
 extern void SetAbortDownloads(bool value);
 
+/**
+* return the platform dependant engine category
+*/
+extern DownloadEnum::Category getPlatformEngineCat();
+
+
 #endif
