@@ -189,7 +189,7 @@ void BattleMapTab::Update(const wxString& Tag)
 }
 
 
-void BattleMapTab::ReloadMaplist()
+void BattleMapTab::ReloadMapList()
 {
 	if (!m_battle)
 		return;
@@ -208,7 +208,8 @@ void BattleMapTab::UpdateUser(User& user)
 	if (&m_battle->GetMe() == &user) {
 		try {
 			m_minimap->UpdateMinimap();
-		} catch (...) {
+		} catch (const std::exception& e) {
+			wxLogWarning(_T("Exception: %s"), e.what());
 		}
 	}
 }
@@ -221,7 +222,8 @@ void BattleMapTab::SetMap(int index)
 	try {
 		m_battle->SetLocalMap(STD_STRING(m_map_combo->GetString(index)));
 		m_battle->SendHostInfo(IBattle::HI_Map);
-	} catch (...) {
+	} catch (const std::exception& e) {
+		wxLogWarning(_T("Exception: %s"), e.what());
 	}
 }
 
@@ -233,7 +235,8 @@ void BattleMapTab::OnMapSelect(wxCommandEvent& /*unused*/)
 	if (!m_battle->IsFounderMe()) {
 		try {
 			m_battle->DoAction("suggests " + STD_STRING(m_map_combo->GetString(m_map_combo->GetCurrentSelection())));
-		} catch (...) {
+		} catch (const std::exception& e) {
+			wxLogWarning(_T("Exception: %s"), e.what());
 		}
 		return;
 	}
@@ -275,7 +278,7 @@ void BattleMapTab::OnUnitsyncReloaded(wxCommandEvent& /*data*/)
 {
 	if (!m_battle)
 		return;
-	ReloadMaplist();
+	ReloadMapList();
 }
 
 void BattleMapTab::SetBattle(IBattle* battle)
@@ -294,7 +297,7 @@ void BattleMapTab::SetBattle(IBattle* battle)
 	if (isBattleEnabled) {
 		m_minimap->SetReadOnly(!m_battle->IsFounderMe());
 		m_start_radios->Enable(m_battle->IsFounderMe());
-		ReloadMaplist();
+		ReloadMapList();
 		Update();
 	}
 }
