@@ -12,8 +12,7 @@ class IServer;
 typedef int Sockerror;
 typedef int Protocolerror;
 
-struct MessageSpamCheck
-{
+struct MessageSpamCheck {
 	time_t lastmessage;
 	unsigned int count;
 };
@@ -44,14 +43,15 @@ public:
 	void OnMotd(const std::string& msg);
 	void OnPong(wxLongLong ping_time);
 
-	void OnNewUser(const std::string& nick, const std::string& conutry, int cpu, int id);
+	void OnNewUser(const std::string& nick, const std::string& country, int id, const std::string& clientAgent);
 	void OnUserStatus(const std::string& nick, UserStatus status);
 	void OnUserQuit(const std::string& nick);
 
 	void OnBattleOpened(int id, BattleType type, NatType nat, const std::string& nick,
 			    const std::string& host, int port, int maxplayers,
 			    bool haspass, int rank, const std::string& maphash, const std::string& engineName, const std::string& engineVersion, const std::string& map,
-			    const std::string& title, const std::string& mod);
+			    const std::string& title, const std::string& mod,
+			    const std::string& channelName);
 
 	void OnUserJoinedBattle(int battleid, const std::string& nick, const std::string& userScriptPassword);
 	void OnUserLeftBattle(int battleid, const std::string& nick);
@@ -83,7 +83,7 @@ public:
 	void OnChannelSaid(const std::string& channel, const std::string& who, const std::string& message);
 	void OnChannelJoin(const std::string& channel, const std::string& who);
 	void OnChannelPart(const std::string& channel, const std::string& who, const std::string& message);
-	void OnChannelTopic(const std::string& channel, const std::string& who, const std::string& message, int when);
+	void OnChannelTopic(const std::string& channel, const std::string& who, const std::string& message);
 	void OnChannelAction(const std::string& channel, const std::string& who, const std::string& action);
 	void OnChannelList(const std::string& channel, const int& numusers, const std::string& topic);
 	void OnUserJoinChannel(const std::string& channel, const std::string& who);
@@ -92,7 +92,7 @@ public:
 	void OnPrivateMessageEx(User& chan, User& who, const std::string& action);
 
 	void OnRequestBattleStatus(int battleid);
-	void OnSaidBattle(int battleid, const std::string& nick, const std::string& msg);
+	void OnBattleSaid(int battleid, const std::string& nick, const std::string& msg);
 	void OnBattleAction(int battleid, const std::string& nick, const std::string& msg);
 
 	void OnAcceptAgreement(const std::string& agreement);
@@ -104,6 +104,11 @@ public:
 	void OnServerMessageBox(const std::string& message);
 	void OnChannelMessage(const std::string& channel, const std::string& msg);
 
+	void OnJoinedFrom(const std::string& channel, const std::string& nick);
+	void OnLeftFrom(const std::string& channel, const std::string& nick);
+	void OnSaidFrom(const std::string& channel, const std::string& nick, const std::string& msg);
+
+
 	void OnHostExternalUdpPort(const unsigned int udpport);
 
 	void OnMyExternalUdpSourcePort(const unsigned int udpport);
@@ -115,9 +120,6 @@ public:
 
 	void OnRedirect(const std::string& address, unsigned int port, const std::string& CurrentNick, const std::string& CurrentPassword);
 
-	/// use this function to check spam from clients and autokick from the battle
-	void AutoCheckCommandSpam(IBattle& battle, User& nick);
-
 	void OnMutelistBegin(const std::string& channel);
 	void OnMutelistItem(const std::string& channel, const std::string& mutee, const std::string& description);
 	void OnMutelistEnd(const std::string& channel);
@@ -132,6 +134,9 @@ public:
 	void OnLoginDenied(const std::string& reason);
 	void OnInvalidFingerprintReceived(const std::string& fingerprint, const std::string& expected_fingerprint);
 
+	/// use this function to check spam from clients and autokick from the battle
+	void AutoCheckCommandSpam(IBattle& battle, User& nick);
+
 private:
 	IServer& m_serv;
 	std::map<std::string, MessageSpamCheck> m_spam_check;
