@@ -4,21 +4,19 @@
 /**
 !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
 
-DO NOT CHANGE THIS FILE!
-
-this file is deprecated and will be replaced with
+TODO: remove all wx stuff & merge / move to
 
 lsl/spring/springprocess.cpp
 
 !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
 **/
+#include "springprocess.h"
 
 #include <wx/log.h>
 
-#include "springprocess.h"
+#include "log.h"
 #include "spring.h"
 #include "utils/platform.h"
-#include "log.h"
 
 DEFINE_LOCAL_EVENT_TYPE(wxEVT_SPRING_EXIT)
 
@@ -47,13 +45,13 @@ void SpringProcess::OnExit()
 {
 	slLogDebugFunc("");
 	wxCommandEvent event(wxEVT_SPRING_EXIT, PROC_SPRING);
-	event.SetExtraLong(m_exit_code);
+	event.SetInt(m_exit_code);
 	m_sp.AddPendingEvent(event);
 }
 
 void* SpringProcess::Entry()
 {
 	slLogDebugFunc("");
-	RunProcess(m_cmd, m_params);
+	m_exit_code = RunProcess(m_cmd, m_params);
 	return NULL;
 }
