@@ -18,7 +18,7 @@
 
 using std::string;
 
-#ifdef WIN32
+#ifdef _WIN32
 #define DELIM "\\"
 #else
 #define DELIM "/"
