@@ -6,9 +6,7 @@
 /**
 !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
 
-DO NOT CHANGE THIS FILE!
-
-this file is deprecated and will be replaced with
+TODO: remove all wx stuff & merge / move to
 
 lsl/container/userlist.h
 
@@ -30,9 +28,8 @@ public:
 	typedef user_map_t::const_iterator user_const_iter_t;
 
 	UserList();
-	virtual ~UserList()
-	{
-	}
+	~UserList();
+
 	void AddUser(User& user);
 	void RemoveUser(std::string const& nick);
 	User& GetUser(std::string const& nick) const;
@@ -41,6 +38,10 @@ public:
 	user_map_t::size_type GetNumUsers() const;
 
 	void Nullify();
+	const user_map_t& GetUsers() const
+	{
+		return m_users;
+	}
 	/*
 	UserList& operator= (const UserList& other) = delete;
 	UserList& operator= (const UserList&& other);
