@@ -4,9 +4,9 @@
 #define SPRINGLOBBY_HEADERGUARD_MIXINS_HH
 
 #include <boost/noncopyable.hpp>
-
-namespace SL {
-	typedef boost::noncopyable  NonCopyable;
+namespace SL
+{
+typedef boost::noncopyable NonCopyable;
 }
 
 #endif // SPRINGLOBBY_HEADERGUARD_MIXINS_HH
